: # use perl
eval '(exit $?0)' && eval 'exec perl -S $0 ${1+"$@"}'
  & eval 'exec perl -S $0 $argv:q'
  if 0;

#
# Licensed Materials - Property of IBM
# Rational ClearQuest 
# (C) Copyright IBM Corp. 2007, 2009 All Rights Reserved
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
##############################################################################

=head1 NAME

cqtssetup.pl

=head1 SYNOPSIS

B<cqtssetup.pl> B<[-man]> B<[-help]>

=head1 OPTIONS

=over 4

=item -help

Print a brief help message.

=item -man

Print the man page.

=back

=head1 DESCRIPTION

=head1 AUTHOR

=cut

use CQPerlExt;

use strict;
use warnings;

# DO NOT TOUCH!!!  Modifying any of these values could stop ClearQuest from working properly.
my $entityCreatedDateFieldName = "RATL_CQ_ENTITY_FIRST_CREATED_DATE";
my $entityLastModifiedDateFieldName = "RATL_CQ_ENTITY_LAST_MODIFIED_DATE";
my $uniqueFieldName = "RATL_CQ_UNIQUE_FIELD";
my $allFieldsName = "RATL_CQ_ALL_FIELDS";


##############################################################################
sub Usage
{
	print <<EOF;

This script is a used to create CQ Full-Text Search configuration files.

Usage: cqperl cqtssetup.pl -s <dbset>
                           -d <databasename>
                           -u <user>
                           -p <password>
                           -o <Entity | CQ | Solr>
                           [-f <input_file>]
                           [-g]

Parameters:
  -s    A ClearQuest database set name
  -d    The ClearQuest user database name
  -u    Login name of the dbset
  -p    Password of the login name
  -o    The operation to be taken; must be "Solr", "CQ" or "Entity"
  -f    Optional 'Entity' file name.  Used only when "Solr" or "CQ" 
        operation is executing
  -g    Index users and groups table.  All clients using this CQ DB
        must be at 8.0.1.04 version or higher.
        
This script is used to generate CQ Full-Text Search configuration files.  
There are three run modes (or operations) that this command provides.  
One operation is to list all entities and their fields of a CQ database -- 
this is known as 'Entity' operation.  A second operation is to generate the 
Search Engine's configuration files (in this case Solr) -- this is known 
as 'Solr' operation.  And a third operation is to generate the CQ's 
configuration files -- this is known as 'CQ' operation.  The last two 
operations, 'Solr' and 'CQ' use the output from the 'Entity' operation to 
generate their output.

Example 1.  Generate a file listing all EntityDefs without users and groups and
            their Fields in the SAMPL database.  The generated file name will 
            be called: Entity-SAMPL-TextSearch.txt
  cqperl cqtssetup.pl -s TextSarch -d SAMPL -u admin -p "" -o Entity
  
Example 2.  Generate a file listing all EntityDefs with users and groups and
            their Fields in the SAMPL database.  The generated file name will 
            be called: Entity-SAMPL-TextSearch.txt
  cqperl cqtssetup.pl -s TextSarch -d SAMPL -u admin -p "" -o Entity -g

Example 3.  Generate the CQ configuration file to be used by CQ Full-Text 
            Search listing all EntityDefs and their Fields to be indexed and 
            searched.  The generated file name will be called:
            CQ-SAMPL-TextSearch.txt
  cqperl cqtssetup.pl -s TextSarch -d SAMPL -u admin -p "" -o CQ -f Entity-SAMPL-TextSearch.txt

Example 4.  Generate the Search Engine configuration file to be used by 
            Full-Text Search Engine (Solr in this case) listing all fields 
            to be indexed and searched.  The generated file name will be 
            called: Solr-SAMPL-TextSearch.txt
  cqperl cqtssetup.pl -s TextSarch -d SAMPL -u admin -p "" -o Solr -f Entity-SAMPL-TextSearch.txt

EOF
	exit 1;
}


if ((@ARGV == 0) || ($ARGV[0] =~ /^-[h\?]/i))
{
	Usage();
	exit 1;
}

require "CQPerlExt.pm" || die;


##############################################################################
# Process command line and setup the envirement
#

my %usage = ();


$usage{s} = "-";	# dbset
$usage{d} = "-";	# databasename
$usage{u} = "-";	# user
$usage{p} = "-";	# password
$usage{o} = "-";	# Solr | CQ | Entity
$usage{f} = "";		# file
$usage{g} = "";     # includeUsersGroups


my $validArgs = 10;

if (($#ARGV + 1) < $validArgs)
{
	print "Invalid number of argumetns ...\n\n";
	Usage();
	exit 1;
}


# Process the command line switches
for (my $switch = 0; $switch < $validArgs; $switch += 2)
{
	my ($d, $switchValue) = split("-", $ARGV[$switch]);
	
	if ($usage{$switchValue})
	{
		$usage{$switchValue} = $ARGV[$switch + 1];
	}
	else
	{
		print "Invalid option \"$ARGV[$switch]\" ...\n\n";
		Usage();
		exit 1;
	}
}


for (my $inputCount = $validArgs; $inputCount <= $#ARGV; $inputCount++)
{
	if ($ARGV[$inputCount] eq "-f")
	{
		# Read user input arguments from the input file
		$usage{"f"} = $ARGV[$inputCount + 1];
		$inputCount++;		
	}
	if ($ARGV[$inputCount] eq "-g")
	{
		$usage{"g"} = "includeUsersGroups";	
	}
}

use Fcntl;
use Cwd 'abs_path';

my $path = abs_path(".");

my $dbSet = $usage{"s"};				# -s 7.1
my $dbName = $usage{"d"};				# -d SAMPL
my $user = $usage{"u"};					# -u admin
my $pwd = $usage{"p"};					# -p ""
my $output = $usage{"o"};				# -o Solr | CQ | Entity
my $entityFile = $usage{"f"};			# <-f file-name>
my $includeUsersGroups = $usage{"g"};	# <-g>

# User session, for doing actual work
my $sessionObj;

eval
{
	$sessionObj = CQSession::Build();

	$sessionObj->UserLogon($user, $pwd, $dbName, $dbSet);
};
die "$@\n" if $@; $@ = "";

my $classpath;
my $java;

if ($ENV{'OS'} =~ /Windows/i) {
    $classpath = './cqtsdbcrawler.jar;./cqjni.jar';
    $java = $ENV{'JRE_HOME'}.'\bin\java.exe';
}else {
    $classpath = './cqtsdbcrawler.jar:./cqjni.jar';
    $java = $ENV{'CQ_HOME'}.'/../common/java/jre/bin/java';
}

my $cmd = "\"$java\" -version";
my $cmdout = `$cmd 2>&1`;

# Check return code
if ($? != 0) {
    die "Can not find executable Java\nPlease check if ClearQuest is installed correctly.\n";
}

# Verify java version
my $version5 = 0;
if ($cmdout =~ /^java version "(\d+)\.(\d+)\./) {
    if ($1 > 1) {
        $version5 = 1;
    }
    elsif (($1 == 1) && ($2 >= 5)) {
        $version5 = 1;
    }
}

if ($version5 == 0) {
    die "This utility needs Java version 1.5 or greater.\n";
}

my $encrypt = "\"$java\" -classpath $classpath com.ibm.rational.clearquest.cqtscrawler.common.ConfigGenerator $output $dbSet $dbName $user \"$pwd\" $entityFile $includeUsersGroups";

print "Executing    : $encrypt\n";

system($encrypt);

print "Status       : Completed\n";

exit;
